###############################################################################
#
# (C) Copyright 2018 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################
from everysk.api_resources.api_resource import (
    RetrievableAPIResource,
    ListableAPIResource,
    DeletableAPIResource,
    CreateableAPIResource,
    UpdateableAPIResource
)

class PrivateSecurity(
    RetrievableAPIResource,
    ListableAPIResource,
    DeletableAPIResource,
    CreateableAPIResource,
    UpdateableAPIResource
):
    @classmethod
    def class_name(cls):
        return 'private_security'



###############################################################################
#
# (C) Copyright 2020 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################
from everysk.api_resources.api_resource import APIResource
from everysk import utils

class MarketData(APIResource):

    def refresh(self):
        return selfs

    @classmethod
    def class_name(cls):
        return 'market_data'
    
    @classmethod
    def class_url(cls):
        return '/%s' % cls.class_name()

    @classmethod
    def __call_method(cls, method, **kwargs):
        api_req = utils.create_api_requestor(kwargs)
        url = '%s/%s' % (cls.class_url(), method)
        response = api_req.post(url, kwargs)
        return response

    @classmethod
    def symbolsSearch(cls, **kwargs):
        return cls.__call_method('symbols_search', **kwargs)

    @classmethod
    def symbolsCheck(cls, **kwargs):
    	return cls.__call_method('symbols_check', **kwargs)

    @classmethod
    def symbolsPrice(cls, **kwargs):
    	return cls.__call_method('symbols_price', **kwargs)
    
    @classmethod
    def symbolsRealtimePrice(cls, **kwargs):
    	return cls.__call_method('symbols_real_time_prices', **kwargs)

    @classmethod
    def symbolsHistorical(cls, **kwargs):
    	return cls.__call_method('symbols_historical', **kwargs)


###############################################################################
#
# (C) Copyright 2020 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################
from everysk.api_resources.api_resource import APIResource
from everysk import utils

class Calculation(APIResource):

    def refresh(self):
        return self

    @classmethod
    def class_name(cls):
        return 'calculations'    
    
    @classmethod
    def class_url(cls):
        return '/%s' % cls.class_name()

    @classmethod
    def __call_method(cls, method, **kwargs):
        api_req = utils.create_api_requestor(kwargs)
        url = '%s/%s' % (cls.class_url(), method)
        response = api_req.post(url, kwargs)
        return response

    @classmethod
    def riskAttribution(cls, **kwargs):
    	return cls.__call_method('risk_attribution', **kwargs)

    @classmethod
    def parametricRiskAttribution(cls, **kwargs):
    	return cls.__call_method('parametric_risk_attribution', **kwargs)

    @classmethod
    def stressTest(cls, **kwargs):
    	return cls.__call_method('stress_test', **kwargs)

    @classmethod
    def exposure(cls, **kwargs):
    	return cls.__call_method('exposure', **kwargs)

    @classmethod
    def properties(cls, **kwargs):
    	return cls.__call_method('properties', **kwargs)

    @classmethod
    def backtest(cls, **kwargs):
        return cls.__call_method('backtest', **kwargs)

    @classmethod
    def backtestStatistics(cls, **kwargs):
        return cls.__call_method('backtest_statistics', **kwargs)

    @classmethod
    def aggregations(cls, **kwargs):
        return cls.__call_method('aggregations', **kwargs)

    @classmethod
    def fundamentals(cls, **kwargs):
        return cls.__call_method('fundamentals', **kwargs)

    @classmethod
    def daysToUnwind(cls, **kwargs):
        return cls.__call_method('days_to_unwind', **kwargs)

    @classmethod
    def sensitivity(cls, **kwargs):
        return cls.__call_method('sensitivity', **kwargs)

    @classmethod
    def optimize(cls, **kwargs):
        return cls.__call_method('optimize', **kwargs)

    @classmethod
    def marginalTrackingError(cls, **kwargs):
        return cls.__call_method('marginal_tracking_error', **kwargs)


###############################################################################
#
# (C) Copyright 2020 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################
from everysk import utils

class APIResource(utils.EveryskObject):
    def __init__(self, retrieve_params, params):
        super(APIResource, self).__init__(retrieve_params, params)
        self.__retrieve_params = retrieve_params    
        return

    def refresh(self, **kwargs):
        api_req = utils.create_api_requestor(kwargs)
        url = '%s/%s' % (self.class_url(), self.get('id'))
        kwargs = self.__retrieve_params
        response = api_req.get(url, kwargs)
        data = response[self.class_name()]
        self.update(data)
        self.clear_unsaved_values()
        return self
 
    @classmethod
    def class_name(cls):
        raise NotImplementedError('APIResource is an abstract class.')

    @classmethod
    def class_name_list(cls):
        cn = cls.class_name()
        if cn[-1] in ('s', 'x'):
            cn += 'es'
        elif cn[-1] in ('y'):
            cn = cn[:-1] + 'ies'
        else:
            cn += 's'
        return cn

    @classmethod
    def class_url(cls):
        return '/%s' % cls.class_name_list()

class RetrievableAPIResource(APIResource):

    @classmethod
    def retrieve(cls, id, **kwargs):
        api_req = utils.create_api_requestor(kwargs)
        url = '%s/%s' % (cls.class_url(), id)
        response = api_req.get(url, kwargs)
        return utils.to_object(cls, kwargs, response)

class ListableAPIResource(APIResource):

    @classmethod
    def list(cls, **kwargs):
        api_req = utils.create_api_requestor(kwargs)
        url = cls.class_url()
        response = api_req.get(url, kwargs)
        return utils.to_list(cls, kwargs, response)

    @classmethod
    def auto_paging_iter(cls, **kwargs):
        params = dict(kwargs)
        page = cls.list(**params)
        while True:
            for item in page:
                yield item
            if page.next_page_token() is None:
                return
            params['page_token'] = page.next_page_token()
            page = cls.list(**params)

class DeletableAPIResource(APIResource):

    def delete(self):
        api_req = utils.create_api_requestor()
        url = '%s/%s' % (self.class_url(), self.get('id'))
        workspace = self.get('workspace', None)
        if workspace:
            url = '%s?workspace=%s' % (url, workspace)
        response = api_req.delete(url)
        data = response[self.class_name()]
        self.clear()
        self.update(data)
        self.clear_unsaved_values()
        return self

    @classmethod
    def remove(cls, id, **kwargs):
        api_req = utils.create_api_requestor()
        url = '%s/%s' % (cls.class_url(), id)
        workspace = kwargs.get('workspace', None)
        if workspace:
            url = '%s?workspace=%s' % (url, workspace)
        response = api_req.delete(url)
        data = response[cls.class_name()]
        return utils.to_object(cls, {}, response)

class CreateableAPIResource(APIResource):

    @classmethod
    def create(cls, **kwargs):
        api_req = utils.create_api_requestor(kwargs)
        url = cls.class_url()
        response = api_req.post(url, kwargs)
        return utils.to_object(cls, kwargs, response)

class UpdateableAPIResource(APIResource):

    @classmethod
    def modify(cls, id, **kwargs):
        api_req = utils.create_api_requestor(kwargs)
        url = '%s/%s' % (cls.class_url(), id)
        response = api_req.put(url, kwargs)
        data = response[cls.class_name()]
        return utils.to_object(cls, kwargs, response)

    def save(self, **kwargs):
        api_req = utils.create_api_requestor(kwargs)
        url = '%s/%s' % (self.class_url(), self.get('id'))
        #response = api_req.put(url, self)
        unsaved_values = self.get_unsaved_values()
        response = api_req.put(url, unsaved_values)
        data = response[self.class_name()]
        self.update(data)
        self.clear_unsaved_values()
        return self

class FilterableAPIResource(APIResource):

    @classmethod
    def filter(cls,**kwargs):
        api_req = utils.create_api_requestor(kwargs)
        url = '/%s/filter' % cls.class_name_list()
        response = api_req.post(url, kwargs)
        return utils.to_list(cls, kwargs, response)


###############################################################################
#
# (C) Copyright 2020 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################
from everysk.api_resources.api_resource import (
    RetrievableAPIResource,
    ListableAPIResource,
    DeletableAPIResource
)
from everysk import utils

class WorkerExecution(
    RetrievableAPIResource,
    ListableAPIResource,
    DeletableAPIResource
):
    @classmethod
    def class_name(cls):
        return 'worker_execution'

    @classmethod
    def retrieve(cls, **kwargs):
        api_req = utils.create_api_requestor(kwargs)
        url = '/workflows%s' % (cls.class_url())
        response = api_req.get(url, kwargs)
        return utils.to_object(WorkerExecution, kwargs, response)




###############################################################################
#
# (C) Copyright 2020 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################
import os
import sys
from everysk import six
import json
import os.path

try:
    import requests
    from requests.packages.urllib3.exceptions import InsecureRequestWarning
    requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
except ImportError:
    requests = None

if not requests:
    NO_REQUEST_MSG = '''
    Warning: the Everysk library is falling back to
    "urllib2/urllib" because "requests" is not installed.
    urllib2's SSL implementation doesn't verify server
    certificates. For improved security, we suggest installing requests.
    If you have any questions, please contact support@everysk.com.
    (HINT: running "pip install -U requests" should upgrade your
    requests library to the latest version.)\n\n'''
    sys.stderr.write(NO_REQUEST_MSG)
    
    from everysk.six.moves import urllib
    from everysk.six.moves.urllib.parse import urlparse    


class HTTPClient(object):
    def __init__(self, timeout=60, verify_ssl_certs=True, allow_redirects=False):
        self.timeout = timeout
        self.allow_redirects = allow_redirects
        self.verify_ssl_certs = verify_ssl_certs
        return

    def request(self, method, url, headers, params=None, payload=None):
        raise NotImplementedError(
            'HTTPClient subclasses must implement `request`'
        )


class RequestsClient(HTTPClient):
    def __init__(self, timeout=60, verify_ssl_certs=True, allow_redirects=False):
        super(RequestsClient, self).__init__(timeout=timeout, verify_ssl_certs=verify_ssl_certs, allow_redirects=allow_redirects)
        if self.verify_ssl_certs:
            #https://curl.haxx.se/docs/caextract.html
            certs_file_name = 'ca-certificates.crt'
            certs_path = os.path.join(os.path.dirname(__file__), certs_file_name)
            if os.path.exists(certs_path):
                self.verify_ssl_certs = certs_path
            else:
                raise Exception('Error loading %s' % certs_file_name)
        return

    def request(self, method, url, headers, params=None, payload=None):
        
        response = requests.request(
            method,
            url,
            headers=headers,
            params=params,
            json=payload,
            timeout=self.timeout,
            verify=self.verify_ssl_certs,
            allow_redirects=self.allow_redirects
        )
        return (response.status_code, response.content)        


class Urllib2Client(HTTPClient):
    def __init__(self, timeout=60, verify_ssl_certs=True, allow_redirects=False):
        super(Urllib2Client, self).__init__(timeout=timeout, verify_ssl_certs=verify_ssl_certs, allow_redirects=allow_redirects)
        return

    def request(self, method, url, headers, params=None, payload=None):
        url_ = '%s?%s' % (url, urllib.parse.urlencode(params)) if params else url
        payload_ = json.dumps(payload) if payload else None

        req = urllib.request.Request(url_, payload_, headers)
        if method not in ('get', 'post'):
            req.get_method = lambda: method.upper()

        try:
            response = urllib.request.urlopen(req)
            rbody = response.read()
            rcode = response.code
            #headers = dict(response.info())
        except urllib.error.HTTPError as e:
            rcode = e.code
            rbody = e.read()
            #headers = dict(e.info())
        except (urllib.error.URLError, ValueError) as e:
            self.__handle_request_error(e)

        #lh = dict((k.lower(), v) for k, v in dict(headers).items())
        #return rbody, rcode, lh
        return (rcode, rbody)

    def __handle_request_error(self, e):
        msg = (
            'Unexpected error communicating with Everysk. '
            'If this problem persists, let us know at support@everysk.com.'
        )
        msg = msg + "\n\n(Network error: " + str(e) + ")"
        raise Exception(msg)


def new_default_http_client(*args, **kwargs):
    if requests:
        impl = RequestsClient
    else:
        impl = Urllib2Client
    return impl(*args, **kwargs)


###############################################################################
#
# (C) Copyright 2020 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################
import json
from everysk import http_client
from everysk.six import python_2_unicode_compatible

OK_STATUS_CODE = (200, 202, 303)


@python_2_unicode_compatible
class APIError(Exception):
    def __init__(self, code, message):
        super(APIError, self).__init__(message)
        self.__code = code
        self.__message = json.loads(message) if message else message
        return

    def __str__(self):
        if self.__code and self.__message:
            return json.dumps(self.__message, sort_keys=True, indent=2)
            # return '%s -> %s' % (self.__code, self.__message)
        return 'API ERROR'


class APIRequestor(object):
    def __init__(self, api_entry, api_version, api_sid, api_token, verify_ssl_certs):
        if not api_entry:
            raise Exception('Empty api_entry')
        if api_version != 'v2':
            raise Exception('Invalid api_version (supported version: "v2")')
        if not api_sid:
            raise Exception('Invalid api_sid')
        if not api_token:
            raise Exception('Invalid api_token')

        self.headers = {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer %s:%s' % (api_sid, api_token),
            'User-Agent': 'Everysk PythonBindings/%s' % (api_version)
        }
        self.base_url = '%s/%s' % (api_entry, api_version)
        self.client = http_client.new_default_http_client(
            timeout=60,
            verify_ssl_certs=verify_ssl_certs,
            allow_redirects=False
        )
        return

    def get(self, path, params):
        url = '%s%s' % (self.base_url, path)
        code, response = self.client.request(
            'GET',
            url,
            headers=self.headers,
            params=params
        )
        if code not in OK_STATUS_CODE:
            raise APIError(code, response)
        return json.loads(response)

    def post(self, path, payload):
        url = '%s%s' % (self.base_url, path)
        code, response = self.client.request(
            'POST',
            url,
            headers=self.headers,
            payload=payload
        )
        if code not in OK_STATUS_CODE:
            raise APIError(code, response)
        return json.loads(response)

    def delete(self, path):
        url = '%s%s' % (self.base_url, path)
        code, response = self.client.request(
            'DELETE',
            url,
            headers=self.headers
        )
        if code not in OK_STATUS_CODE:
            raise APIError(code, response)
        return json.loads(response)

    def put(self, path, payload):
        url = '%s%s' % (self.base_url, path)
        code, response = self.client.request(
            'PUT',
            url,
            headers=self.headers,
            payload=payload
        )
        if code not in OK_STATUS_CODE:
            raise APIError(code, response)
        return json.loads(response)



from setuptools import setup

setup(name='everysk',
      packages=['everysk', 'everysk.api_resources'],
      package_data={'everysk': ['*.crt']},
      version='1.0.16',
      description='Python client for Everysk API',
      author='Everysk Technologies',
      author_email='contact@everysk.com',
      url='https://github.com/Everysk/Everysk-API-Python',
      install_requires=['requests', 'setuptools'],
      keywords=['everysk'],
      license='MIT',
      zip_safe=False)


###############################################################################
#
# (C) Copyright 2020 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################
import os

api_sid = None
api_token = None
api_entry = 'https://api.everysk.com'
api_version = 'v2'
verify_ssl_certs = True

def get_api_config(params):
    
    api_entry_ = params.get('api_entry', None)
    if api_entry_ is None:
      global api_entry
      api_entry_ = api_entry
      api_entry_ = os.getenv('EVERYSK_API_ENTRY', api_entry_)

    api_version_ = params.get('api_version', None)
    if api_version_ is None:
      global api_version
      api_version_ = api_version

    api_sid_ = params.get('api_sid', None)
    if api_sid_ is None:
      global api_sid
      api_sid_ = api_sid
      api_sid_ = os.getenv('EVERYSK_API_SID', api_sid_)

    api_token_ = params.get('api_token', None)
    if api_token_ is None:
      global api_token
      api_token_ = api_token
      api_token_ = os.getenv('EVERYSK_API_TOKEN', api_token_)

    verify_ssl_certs_ = params.get('verify_ssl_certs', None)
    if verify_ssl_certs_ is None:
      global verify_ssl_certs
      verify_ssl_certs_ = verify_ssl_certs

    return (api_entry_, api_version_, api_sid_, api_token_, verify_ssl_certs_)

from everysk.utils import *
from everysk.api_resources import *



###############################################################################
#
# (C) Copyright 2020 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################
from everysk.api_resources.api_resource import (
    RetrievableAPIResource,
    ListableAPIResource,
    DeletableAPIResource,
    UpdateableAPIResource
)
from everysk import utils

class Report(
    RetrievableAPIResource,
    ListableAPIResource,
    DeletableAPIResource,
    UpdateableAPIResource
):
    @classmethod
    def class_name(cls):
        return 'report'  

    def share(self, **kwargs):
        api_req = utils.create_api_requestor(kwargs)
        url = '%s/%s/share' % (self.class_url(), self.get('id'))
        response = api_req.post(url, kwargs)
        data = response[self.class_name()]
        self.update(data)
        self.clear_unsaved_values()
        return self



###############################################################################
#
# (C) Copyright 2018 EVERYSK TECHNOLOGIES
#
# This is an unpublished work containing confidential and proprietary
# information of EVERYSK TECHNOLOGIES. Disclosure, use, or reproduction
# without authorization of EVERYSK TECHNOLOGIES is prohibited.
#
###############################################################################
from everysk.api_resources.api_resource import (
    RetrievableAPIResource,
    ListableAPIResource,
    DeletableAPIResource,
    CreateableAPIResource,
    UpdateableAPIResource,
    FilterableAPIResource
)
from everysk import utils

class Datastore(
    RetrievableAPIResource,
    ListableAPIResource,
    DeletableAPIResource,
    CreateableAPIResource,
    UpdateableAPIResource,
    FilterableAPIResource
):
    @classmethod
    def class_name(cls):
        return 'datastore'

    @classmethod
    def explore(cls, **kwargs):
        api_req = utils.create_api_requestor(kwargs)
        url = '/%s/explore' % cls.class_name_list()
        response = api_req.post(url, kwargs)
        return response